@@ -714,7 +714,7 @@ public:
     {
         Surface surface{client};
 
-        wl_shell_surface * shell_surface = wl_shell_get_shell_surface(shell, surface);
+        wl_shell_surface * shell_surface = wl_shell_get_shell_surface(the_shell(), surface);
         surface.run_on_destruction([shell_surface]()
             {
                 wl_shell_surface_destroy(shell_surface);
@@ -768,9 +768,43 @@ public:
         return surface;
     }
 
+    Surface create_visible_surface(Client& client, int width, int height)
+    {
+        if (shell)
+        {
+            return create_wl_shell_surface(client, width, height);
+        }
+        else if (xdg_shell_stable)
+        {
+            return create_xdg_shell_stable_surface(client, width, height);
+        }
+        else if (xdg_shell_v6)
+        {
+            return create_xdg_shell_v6_surface(client, width, height);
+        }
+        else
+        {
+            throw std::runtime_error("compositor does not support any known shell protocols");
+        }
+    }
+
     wl_shell* the_shell() const
     {
-        return shell;
+        if (shell)
+        {
+            return shell;
+        }
+        else
+        {
+            if (!supported_extensions || !supported_extensions->count("wl_shell"))
+            {
+                BOOST_THROW_EXCEPTION((ExtensionExpectedlyNotSupported{"wl_shell", AnyVersion}));
+            }
+            else
+            {
+                throw std::runtime_error("Failed to bind to wl_shell");
+            }
+        }
     }
 
     zxdg_shell_v6* the_xdg_shell_v6() const
@@ -1635,7 +1669,7 @@ wlcs::Surface wlcs::Client::create_xdg_shell_stable_surface(int width, int heigh
 
 wlcs::Surface wlcs::Client::create_visible_surface(int width, int height)
 {
-    return impl->create_wl_shell_surface(*this, width, height);
+    return impl->create_visible_surface(*this, width, height);
 }
 
 size_t wlcs::Client::output_count() const
